' Add a COM Reference to Microsoft Shell Controls and Automation
Imports System.Threading
Imports System.Text.RegularExpressions
Imports System.Collections.Specialized
Imports Microsoft.Win32
Imports System.ComponentModel

#Const SLEEP_DELAY = 200
'
' A GUI-based "management console" for performing common administrative duties
' for PCs in a LAN.  Draws heavily upon the SOSOS database and other SOSOS
' related programs.
'
' Note: Many features may be unique to my organization, and may require some
' tweaking to suit your needs.
'
Public Class fmMain

    Private dt As DataTable
    Private CurrentFont As Font
    ' Regular Expression syntax is still a black art...
    Private RegExPattern As String = "(?<Org>\w+)\s*(?<Div>\w+)\s*(?<Name>[^,]*).*,\s*Bldg\s*(?<Bldg>\w+)"
    Private Is_Alive_Column As Integer


    Private Sub fmMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        Dim sb As New System.Text.StringBuilder
        Dim xw As System.Xml.XmlWriter

        If Not IsNothing(dt) AndAlso dt.Rows.Count > 0 Then
            ' save the existing data
            xw = System.Xml.XmlWriter.Create(sb)
            dt.WriteXml(xw)
            My.Settings.CachedData = sb.ToString
        End If
    End Sub

    Private Sub fmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim XML_DataTable As String
        Dim il_large As New ImageList
        Dim il_small As New ImageList
        Dim fc As New FontConverter
        Dim pk(0) As DataColumn
        Dim maxThreads, portThreads As Integer

        Dim currentDomain As AppDomain = AppDomain.CurrentDomain
        'for regular unhandled stuff 
        AddHandler currentDomain.UnhandledException, AddressOf MYExceptionHandler
        'for threads behind forms 
        AddHandler Application.ThreadException, AddressOf MYThreadHandler

        Me.Size = My.Settings.WindowSize

        CurrentFont = CType(fc.ConvertFromString(My.Settings.DefaultFont), Font)
        Me.Font = CurrentFont

        ' Let's pick some icons
        il_large.ColorDepth = ColorDepth.Depth16Bit
        il_large.ImageSize = New Size(64, 64)
        il_large.Images.Add(My.Resources.PC_Online)
        il_large.Images.Add(My.Resources.PC_Offline)
        il_large.Images.Add(My.Resources.Laptop_Online)
        il_large.Images.Add(My.Resources.Laptop_Offline)
        il_large.Images.Add(My.Resources.Server_Online)
        il_large.Images.Add(My.Resources.Server_Offline)

        il_small.ColorDepth = ColorDepth.Depth16Bit
        il_small.ImageSize = New Size(16, 16)
        il_small.Images.Add(My.Resources.PC_Online_small)
        il_small.Images.Add(My.Resources.PC_Offline_small)
        il_small.Images.Add(My.Resources.Laptop_Online_small)
        il_small.Images.Add(My.Resources.Laptop_Offline_small)
        il_small.Images.Add(My.Resources.Server_Online_small)
        il_small.Images.Add(My.Resources.Server_Offline_small)

        ' associate the imagelist with the listview control
        ListView1.LargeImageList = il_large
        ListView1.SmallImageList = il_small
        ListView1.View = View.LargeIcon
        ListView1.AllowColumnReorder = True

        ' Create columns for the items and subitems.
        ListView1.Columns.Add("PC", 150, HorizontalAlignment.Left)
        ListView1.Columns.Add("Comment", 200, HorizontalAlignment.Left)
        ListView1.Columns.Add("Model", 150, HorizontalAlignment.Left)
        ListView1.Columns.Add("OS", 175, HorizontalAlignment.Left)
        ListView1.Columns.Add("Bldg", 50, HorizontalAlignment.Right)
        ListView1.Columns.Add("Div", 50, HorizontalAlignment.Left)
        ListView1.Columns.Add("Date", 130, HorizontalAlignment.Left)
        ListView1.Columns.Add("IsAlive", 50, HorizontalAlignment.Left)
        Is_Alive_Column = 7

        ' Next, we create a detached local datatable to store the values
        ' used in the ListView (and other fields used internally)
        dt = New DataTable
        dt.TableName = "Summary"
        dt.Columns.Add(New DataColumn("ID", GetType(System.String)))
        dt.Columns.Add(New DataColumn("PC_Name", GetType(System.String)))
        dt.Columns.Add(New DataColumn("PC_Comment", GetType(System.String)))
        dt.Columns.Add(New DataColumn("PC_Model", GetType(System.String)))
        dt.Columns.Add(New DataColumn("OS", GetType(System.String)))
        dt.Columns.Add(New DataColumn("Bldg", GetType(System.String)))
        dt.Columns.Add(New DataColumn("Division", GetType(System.String)))
        dt.Columns.Add(New DataColumn("CurrentDate", GetType(System.DateTime)))
        dt.Columns.Add(New DataColumn("Icon", GetType(System.Int32)))
        dt.Columns.Add(New DataColumn("IsAlive", GetType(System.Boolean)))
        ' make PC_Name the primary key
        pk(0) = dt.Columns(1)
        dt.PrimaryKey = pk

        ' set the maxiumum number of threads in the pool
        ThreadPool.GetMaxThreads(maxThreads, portThreads)
        ThreadPool.SetMaxThreads(CInt(My.Settings.PoolSize), portThreads)

        If My.Settings.ListViewView = "Icon" Then
            tsmiViewIcon.PerformClick()
        Else
            tsmiViewDetails.PerformClick()
        End If

        ' Since the generation of the initial data is kinda slow and the
        ' the list of PCs rarely change, we use previously cached data when
        ' available.
        XML_DataTable = My.Settings.CachedData
        If XML_DataTable <> "" Then
            Dim sr As New System.IO.StringReader(XML_DataTable)
            Dim dr As DataRow
            Dim i As Integer
            Dim lvi As ListViewItem
            dt.ReadXml(sr)

            ListView1.BeginUpdate()
            ListView1.Items.Clear()
            For Each dr In dt.Rows
                dr("IsAlive") = False

                i = CInt(dr("Icon"))
                ' remove the Alive icon from the cached data
                If i Mod 2 = 0 Then
                    i += 1
                End If
                dr("Icon") = i

                lvi = ListView1.Items.Add(dr("PC_Name").ToString, i)
                lvi.SubItems.Add(dr("PC_Comment").ToString)
                lvi.SubItems.Add(dr("PC_Model").ToString)
                lvi.SubItems.Add(dr("OS").ToString)
                lvi.SubItems.Add(dr("Bldg").ToString)
                lvi.SubItems.Add(dr("Division").ToString)
                lvi.SubItems.Add(dr("CurrentDate").ToString)
                lvi.SubItems.Add(dr("IsAlive").ToString)
            Next
            ListView1.Sorting = SortOrder.Ascending
            ListView1.Sort()
            ListView1.EndUpdate()

            tsslTotalPCs.Text = "Total PCs: " & dt.Rows.Count
        Else
            ' change the text to "Generate" rather than "Regenerate"
            tsbRegenerate.Text = "Generate List"
            tsmiDataRegenerate.Text = "Generate list of PCs"
        End If
    End Sub

    '
    ' Generate the list of PCs that appear in the Management Console
    '
    Private Sub Regenerate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbRegenerate.Click, tsmiDataRegenerate.Click
        Dim SQL, PC_List(), PC_Name, buf As String
        Dim lvi As ListViewItem
        Dim dr, drs() As DataRow
        Dim dt_sosos As New DataTable
        Dim db As Database
        Dim ChassisTypes() As String = {"Portable", "Laptop", "Notebook", "Hand Held", "Docking Station", "All in One", "Sub Notebook"}
        Dim regx As Regex
        Dim m As Match

        tsslStatus.Text = "Status: Regenerating..."
        Application.DoEvents()
        Me.Cursor = Cursors.WaitCursor
        tspbStatus.Maximum = 4

        ' create a regular expression object for the Comment field
        regx = New Regex(RegExPattern, RegexOptions.Compiled Or RegexOptions.IgnoreCase)

        ' pull a list of PCs as the "base list"
        Try
            Select Case My.Settings.GenerateMethod
                Case "AD"
                    Dim net As New GetNetList

                    PC_list = net.GetPCOUList(My.Settings.Domain, My.Settings.OU_Filter, My.Settings.NameFilter, False)
                Case "Browse"
                    Dim Temp, TempList() As String
                    Dim sc As New StringCollection
                    Dim net As New GetNetList

                    ' If browsing is your poll method, then that means that there
                    ' is no "base list" of PCs, so we combine the lists of PCs from
                    ' the browse list and SOS database 
                    TempList = net.GetPCBrowseList(My.Settings.Domain, My.Settings.NameFilter)
                    For Each Temp In TempList
                        sc.Add(Temp)
                    Next

                    For Each Temp In TempList
                        If sc.Contains(Temp) = False Then
                            sc.Add(Temp)
                        End If
                    Next

                    ReDim PC_list(sc.Count - 1)
                    sc.CopyTo(PC_list, 0)
                    Array.Sort(PC_list)
                Case "NT"
                    Dim net As New GetNetList

                    PC_list = net.GetPCDomList(My.Settings.Domain, My.Settings.NameFilter)
                Case Else
                    Throw New ApplicationException("Generate Method " & My.Settings.GenerateMethod & " is not supported")
            End Select
        Catch ex As Exception
            MsgBox("Can't get the list of PCs" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
            tsslStatus.Text = "Status: Ready"
            Me.Cursor = Cursors.Default
            Exit Sub
        End Try

        tspbStatus.Increment(1)

        ' Start with some default values
        dt.Clear()
        For Each PC_Name In PC_list
            dr = dt.NewRow
            dr("PC_Name") = PC_Name
            dr("Icon") = 1
            dr("IsAlive") = False
            dt.Rows.Add(dr)
        Next

        ' Now, query the SOSOS database to populate the remaining fields in
        ' the database
        SQL = "SELECT ID, PC_Name, PC_Model, SOS.PC_Comment, CurrentDate, OS, PC_ChassisType "
        SQL &= "FROM SOS LEFT JOIN "
        SQL &= "PC ON PC.ID_PC = SOS.ID"

        dt_sosos.Clear()
        Try
            db = New Database
            db.Fill(SQL, dt_sosos)
        Catch ex As Exception
            MsgBox("Can't connect to the SOSOS database" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
            Exit Sub
        End Try
        tspbStatus.Increment(1)

        ' merge the two lists together
        For Each dr In dt.Rows
            drs = dt_sosos.Select("PC_Name='" & dr("PC_Name").ToString & "'")
            If UBound(drs) >= 0 Then
                dr("ID") = drs(0)("ID")
                dr("PC_Comment") = drs(0)("PC_Comment")
                dr("PC_Model") = drs(0)("PC_Model")
                dr("OS") = drs(0)("OS")

                ' Use regular expressions to parse the commnent for a Divison
                ' and Building field.  Obviously this is somewhat unique to
                ' my organization, where the PC's Comment looks like this:
                ' Organization Division UserName [, Shop], Bldg 12345
                '
                m = regx.Match(dr("PC_Comment").ToString)
                If m.Success Then
                    dr("Division") = m.Groups("Div").Value
                    dr("Bldg") = m.Groups("Bldg").Value
                End If

                dr("CurrentDate") = drs(0)("CurrentDate")
                If dr("OS").ToString.Contains("Server") Then
                    dr("Icon") = 5
                End If

                ' Use the PC_ChassisType to determine if we should use the
                ' laptop icon
                buf = drs(0)("PC_ChassisType").ToString.Split(","c)(0).Trim
                If Array.IndexOf(ChassisTypes, buf) >= 0 Then
                    dr("Icon") = 3
                End If
            End If
        Next
        tspbStatus.Increment(1)

        ' fill the listview
        ListView1.BeginUpdate()
        ListView1.Items.Clear()
        For Each dr In dt.Rows
            lvi = ListView1.Items.Add(dr("PC_Name").ToString, CInt(dr("Icon")))
            lvi.SubItems.Add(dr("PC_Comment").ToString)
            lvi.SubItems.Add(dr("PC_Model").ToString)
            lvi.SubItems.Add(dr("OS").ToString)
            lvi.SubItems.Add(dr("Bldg").ToString)
            lvi.SubItems.Add(dr("Division").ToString)
            lvi.SubItems.Add(dr("CurrentDate").ToString)
            lvi.SubItems.Add(dr("IsAlive").ToString)
        Next
        ListView1.Sorting = SortOrder.Ascending
        ListView1.Sort()
        ListView1.EndUpdate()

        tsslStatus.Text = "Status: Ready"
        tsslTotalPCs.Text = "Total PCs: " & PC_list.Length.ToString
        Me.Cursor = Cursors.Default
        tspbStatus.Value = 0
    End Sub

    '
    ' Select the Icon View
    ' Note: this is a per-user setting
    '
    Private Sub tsmiViewIcon_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiViewIcon.Click
        ' reduce the font size a wee bit in the Large Icon view
        ListView1.Font = New Font(CurrentFont.Name, CSng(CurrentFont.Size * 0.7))
        ListView1.View = View.LargeIcon
        tsmiViewIcon.Checked = True
        tsmiViewDetails.Checked = False
        My.Settings.ListViewView = "Icon"
    End Sub

    '
    ' Select the Details View
    ' Note: this is a per-user setting
    '
    Private Sub tsmiViewDetails_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiViewDetails.Click
        ListView1.Font = CurrentFont
        ListView1.View = View.Details
        tsmiViewIcon.Checked = False
        tsmiViewDetails.Checked = True
        My.Settings.ListViewView = "Details"
    End Sub

    '
    ' Just a convenient way to package up all of the data that we need to
    ' send to the delegate for the ThreadPool
    '
    Private Class pkg
        Public waitHandle As AutoResetEvent
        Public RemotePC As String
        Public IsAlive As Boolean
    End Class

    '
    ' Start the Ping Validation process.  Since this takes a while to complete,
    ' we use a thread pool to "spool up" the tasks to be run concurrently.
    '
    Private Sub PingValidate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbPingValidate.Click, tsmiDataPingValidate.Click
        Dim i, AliveCount As Integer
        Dim PC_Name As String
        Dim pkg() As pkg
        Dim lvi As ListViewItem
        Dim dr, drs() As DataRow

        If dt.Rows.Count = 0 Then
            Beep()
            Exit Sub
        End If

        tsslStatus.Text = "Status: Validating..."
        Application.DoEvents()
        Me.Cursor = Cursors.WaitCursor
        tspbStatus.Maximum = dt.Rows.Count

        ReDim pkg(dt.Rows.Count - 1)
        i = 0
        For Each dr In dt.Rows
            ' Setup the "package" that contains the information
            ' we need to send to the delegate
            pkg(i) = New pkg
            pkg(i).waitHandle = New AutoResetEvent(False)
            pkg(i).RemotePC = dr("PC_Name").ToString
            pkg(i).IsAlive = False

            ' put it in the queue
            ThreadPool.QueueUserWorkItem(AddressOf DoPing, pkg(i))
            i += 1
        Next

        For i = 0 To UBound(pkg)
            ' Wait until the task finishes or reaches the maximum timeout value
            pkg(i).waitHandle.WaitOne(2000, False)

            ' change values for the icon and IsAlive columns
            dr = dt.Rows(i)
            PC_Name = dr("PC_Name").ToString
            lvi = ListView1.FindItemWithText(PC_Name)

            If CBool(dr("IsAlive")) <> pkg(i).IsAlive Then
                dr("IsAlive") = pkg(i).IsAlive
                If pkg(i).IsAlive Then
                    dr("Icon") = CInt(dr("Icon")) - 1
                Else
                    dr("Icon") = CInt(dr("Icon")) + 1
                End If
                lvi.ImageIndex = CInt(dr("Icon"))
                lvi.SubItems.Item(Is_Alive_Column).Text = dr("IsAlive").ToString
                Application.DoEvents()
            End If
            tspbStatus.Increment(1)
        Next

        ' a quick way to count the number of PCs that are alive
        drs = dt.Select("IsAlive=True")
        AliveCount = UBound(drs) + 1

        tsslOnLinePCs.Text = "PCs Online: " & AliveCount.ToString
        tsslLastValidation.Text = "Validated: " & Now
        tsslStatus.Text = "Status: Ready"
        Me.Cursor = Cursors.Default
        tspbStatus.Value = 0
    End Sub

    '
    ' Perform the Ping Validation inside the threadpool
    '
    Sub DoPing(ByVal State As Object)
        Dim p As New Ping
        Dim pkg As pkg

        pkg = DirectCast(State, pkg)

        Try
            If p.Ping(pkg.RemotePC) <= 1000 Then
                pkg.IsAlive = True
            End If
        Catch ex As Exception
            ' ignore
        End Try

        ' Signal that we're done, and exit
        pkg.waitHandle.Set()
    End Sub

    '
    ' Record the current window size
    ' Note: this is a per-user setting
    '
    Private Sub fmMain_ResizeEnd(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.ResizeEnd
        My.Settings.WindowSize = Me.Size
    End Sub

    '
    ' Allow the user to specify a Font
    ' Note: this is a per-user setting
    '
    Private Sub tsmiViewFont_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiViewFont.Click
        FontDialog1.Font = ListView1.Font
        If FontDialog1.ShowDialog = Windows.Forms.DialogResult.OK Then
            Dim fc As New FontConverter

            CurrentFont = FontDialog1.Font
            Me.Font = CurrentFont

            If tsmiViewIcon.Checked = True Then
                ListView1.Font = New Font(CurrentFont.Name, CSng(CurrentFont.Size * 0.75))
            End If
            ListView1.Refresh()

            My.Settings.DefaultFont = fc.ConvertToString(Me.Font)
        End If
    End Sub

    '
    ' Return the PCName based upon the currently selected ListViewItem
    '
    Private Function GetPCName() As String
        Dim lvi As ListViewItem

        If ListView1.SelectedItems.Count = 1 Then
            lvi = ListView1.SelectedItems(0)
            Return lvi.Text
        End If

        Return ""
    End Function

    '
    ' Return the PC_ID field.  Used to verify that there is a valid record in
    ' the SOSOS database
    '
    Private Function GetPC_ID(ByVal PC As String) As String
        Dim drs() As DataRow
        Dim PC_ID As String

        PC_ID = ""
        drs = dt.Select("PC_Name='" & PC & "'")
        If UBound(drs) >= 0 Then
            PC_ID = drs(0)("ID").ToString
        End If

        Return PC_ID
    End Function

    '
    ' Load the right-click context menu
    '
    Private Sub ListView1_MouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ListView1.MouseClick
        If e.Button = Windows.Forms.MouseButtons.Right Then
            If ListView1.SelectedItems.Count <> 1 Then
                Exit Sub
            End If

            ContextMenuStrip1.Show(Me, New Point(e.X + 2, e.Y + 50))
        End If
    End Sub

    '
    ' Provide context-senstitive tooltips
    '
    Private Sub ListView1_ItemMouseHover(ByVal sender As Object, ByVal e As System.Windows.Forms.ListViewItemMouseHoverEventArgs) Handles ListView1.ItemMouseHover
        Dim drs() As DataRow

        drs = dt.Select("PC_Name='" & e.Item.Text & "'")
        If UBound(drs) >= 0 AndAlso drs(0)("ID").ToString <> "" Then
            ToolTip1.SetToolTip(Me.ListView1, "Comment: " & drs(0)("PC_Comment").ToString & vbCrLf & "Model: " & drs(0)("PC_Model").ToString & vbCrLf & "OS: " & drs(0)("OS").ToString & vbCrLf & "Bldg: " & drs(0)("Bldg").ToString & vbCrLf & "Div: " & drs(0)("Division").ToString)
        Else
            ToolTip1.SetToolTip(Me.ListView1, "No Data")
        End If
    End Sub

    '
    ' Query the SOSOS database for the records for the specified PC and
    ' display the results.
    ' Note: Uses the ViewSOSOS application
    '
    Private Sub tsmiActionQuery_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiActionQuery.Click, cmsiQuery.Click
        Dim PC, PC_ID As String

        PC = GetPCName()
        ' if we get a PC_ID value, then we know there is valid SOSOS data
        ' for this PC
        PC_ID = GetPC_ID(PC)
        If PC_ID = "" Then
            Beep()
            Exit Sub
        End If

        ' start this as a background task so you can have multiple
        ' instances running.
        Dim qs As New QuerySOSOS
        qs.PC = PC

        Dim t As New Thread(AddressOf qs.Doit)
        t.Start()
    End Sub

    '
    ' Gather new SOSOS data for the specified PC and display the results
    ' Note: Uses the RunSOSOS and ViewSOSOS applications
    '
    Private Sub tsmiActionPoll_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionPoll.Click, cmsiPoll.Click
        Dim PC As String

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        ' start this as a background task so you can have multiple
        ' instances running.
        Dim gsd As New GetSOSOSData
        gsd.PC = PC

        Dim t As New Thread(AddressOf gsd.DoIt)
        t.Start()
    End Sub

    '
    ' Return the name of the person logged on to the specified remote PC
    '
    Private Sub tsmiActionGetUser_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionGetUser.Click, cmsiGetUser.Click
        Dim PC As String
        Dim gu As New GetUser

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        Try
            tsslStatus.Text = "Status: Remote User..."
            Application.DoEvents()
            MsgBox(gu.GetRemoteUser(PC))
        Catch ex As Exception
            MsgBox("Can't get current user" & vbCr & ex.Message)
        End Try
        tsslStatus.Text = "Status: Ready"
    End Sub

    '
    ' Ping an specified PC to verify that it's on line
    '
    Private Sub tsmiActionPing_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionPing.Click, cmsiPing.Click
        Dim p As New Ping
        Dim PC As String
        Dim AliveCount, response As Integer
        Dim lvi As ListViewItem
        Dim dr, drs() As DataRow

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        dr = dt.Select("PC_Name='" & PC & "'")(0)
        lvi = ListView1.SelectedItems(0)

        Try
            response = p.Ping(PC)
        Catch ex As Exception
            response = Integer.MaxValue
        End Try

        If response <= 1000 Then
            If CBool(dr("IsAlive")) = False Then
                dr("IsAlive") = True
                dr("Icon") = CInt(dr("Icon")) - 1
                lvi.ImageIndex = CInt(dr("Icon"))
            End If
        Else
            If CBool(dr("IsAlive")) = True Then
                dr("IsAlive") = False
                dr("Icon") = CInt(dr("Icon")) + 1
                lvi.ImageIndex = CInt(dr("Icon"))
            End If
        End If

        ' Update the Alive Count number
        drs = dt.Select("IsAlive=True")
        AliveCount = UBound(drs) + 1

        tsslOnLinePCs.Text = "PCs Online: " & AliveCount.ToString
    End Sub

    '
    ' Launch the registry editor for the specified PC
    '
    Private Sub tsmiActionRegistry_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionRegistry.Click, cmsiRegistry.Click
        Dim PC As String

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        ' start this as a background task so you can have multiple
        ' instances running.
        Dim lr As New LaunchRegedit
        lr.PC = PC

        Dim t As New Thread(AddressOf lr.Doit)
        t.Start()
    End Sub

    '
    ' Launch the Microsoft Managment Console (MMC) for the specified PC
    '
    Private Sub tsmiActionMgmt_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionMgmt.Click, cmsiMgmt.Click
        Dim PC As String

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        ' start this as a background task so you can have multiple
        ' instances running.
        Dim lmmc As New LaunchMMC
        lmmc.PC = PC

        Dim t As New Thread(AddressOf lmmc.Doit)
        t.Start()
    End Sub

    '
    ' Launch the Windows Explorer for the specified PC
    '
    Private Sub tsmiActionExplorer_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionExplorer.Click, cmsiExplorer.Click
        Dim PC As String

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        ' start this as a background task so you can have multiple
        ' instances running.
        Dim le As New LaunchExplorer
        le.PC = PC

        Dim t As New Thread(AddressOf le.Doit)
        t.Start()
    End Sub

    '
    ' Edit the Comment on the specified PC
    '
    Private Sub tsmiActionEdit_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionEdit.Click, cmsiEdit.Click
        Dim reg_hklm, key As RegistryKey
        Dim f As New fmComment
        Dim PC, buf As String

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        buf = ""
        Try
            reg_hklm = RegistryKey.OpenRemoteBaseKey(RegistryHive.LocalMachine, PC)
            key = reg_hklm.OpenSubKey("SYSTEM\CurrentControlSet\Services\lanmanserver\parameters", True)
            buf = key.GetValue("srvcomment", "").ToString
        Catch ex As Exception
            MsgBox("Can't get the Comment field" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
            Exit Sub
        End Try

        f.Comment = buf
        f.PC = PC
        f.ShowDialog()

        If f.Comment <> "" And f.Comment <> buf Then
            Try
                key.SetValue("srvcomment", f.Comment)
                MsgBox("Done!")
            Catch ex As Exception
                MsgBox("Can't set the Comment field" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
            End Try
        End If
        key.Close()
        reg_hklm.Close()
    End Sub

    '
    ' Send a "Net Send" message to the specified PC
    '
    Private Sub tsmiActionSend_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionSend.Click, cmsiSend.Click
        Dim f As New fmMessage
        Dim PC As String

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        Try
            f.PCName = PC
            f.ShowDialog()
            MsgBox("Message Sent")
        Catch ex As Exception
            MsgBox("Could not send message" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub


    '
    ' Log the user off of the specified PC
    '
    Private Sub tsmiActionLogoff_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionLogoff.Click, cmsiLogoff.Click
        Dim PC As String
        Dim a As New Actions

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        Try
            a.DoAction(PC, Actions.ActionType.Logoff, True)
            MsgBox("Logoff successful")
        Catch ex As Exception
            MsgBox("Could not Logoff the user of the PC" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    '
    ' Reboot the specified PC
    '
    Private Sub tsmiActionReboot_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionReboot.Click, cmsiReboot.Click
        Dim PC As String
        Dim a As New Actions

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        Try
            a.DoAction(PC, Actions.ActionType.Reboot, True)
            MsgBox("Reboot successful")
        Catch ex As Exception
            MsgBox("Could not Reboot the PC" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    '
    ' Shutdown the specified PC
    '
    Private Sub tsmiActionShutdown_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionShutdown.Click, cmsiShutdown.Click
        Dim PC As String
        Dim a As New Actions

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        Try
            a.DoAction(PC, Actions.ActionType.Shutdown, True)
            MsgBox("Shutdown successful")
        Catch ex As Exception
            MsgBox("Could not Shutdown the PC" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    '
    ' Launch the Remote Desktop (aka Terminal Service Client) for the 
    ' specifed PC.
    ' Note: Allows for starting the service and adjusting the permissions
    ' to allow remote connections
    '
    Private Sub tsmiActionRemoteDesktop_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiActionRemoteDesktop.Click, cmsiRemoteDesktop.Click
        Dim PC As String

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        ' start this as a background task so you can have multiple
        ' instances running.
        Dim lrdp As New LaunchRDP
        lrdp.PC = PC

        Dim t As New Thread(AddressOf lrdp.Doit)
        t.Start()

    End Sub

    '
    ' Get the idle time of the remote PC.
    ' Note: Requires an application that is installed on the remote PC
    '
    Private Sub tsmiActionGetIdle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiActionGetIdle.Click, cmsiGetIdle.Click
        Dim it As New IdleTime
        Dim PC As String

        PC = GetPCName()
        If PC = "" Then
            Beep()
            Exit Sub
        End If

        Try
            MsgBox(it.GetIdleTime(PC).ToString)
        Catch ex As Exception
            MsgBox("Could not get the Idle Time for the PC" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Private Sub tsmiFileExit_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmiFileExit.Click
        Me.Close()
    End Sub

    Private Sub ListView1_ColumnClick(ByVal sender As Object, ByVal e As System.Windows.Forms.ColumnClickEventArgs) Handles ListView1.ColumnClick
        Static sortColumn As Integer = -1

        ' Determine whether the column is the same as the last column clicked.
        If e.Column <> sortColumn Then
            ' Set the sort order to ascending by default.
            ListView1.Sorting = SortOrder.Ascending
        Else
            ' Determine what the last sort order was and change it.
            If ListView1.Sorting = SortOrder.Ascending Then
                ListView1.Sorting = SortOrder.Descending
            Else
                ListView1.Sorting = SortOrder.Ascending
            End If
        End If

        sortColumn = e.Column
        ' Set the ListViewItemSorter property to a new ListViewItemComparer object.
        ListView1.ListViewItemSorter = New ListViewItemComparer(e.Column, ListView1.Sorting)

        ' Call the sort method to manually sort.
        ListView1.Sort()
    End Sub

    Private Sub MYExceptionHandler(ByVal sender As Object, ByVal e As UnhandledExceptionEventArgs)
        Dim EX As Exception
        EX = CType(e.ExceptionObject, Exception)
        MsgBox(EX.StackTrace)
    End Sub

    Private Sub MYThreadHandler(ByVal sender As Object, ByVal e As Threading.ThreadExceptionEventArgs)
        MsgBox(e.Exception.StackTrace)
    End Sub
End Class

'
' Class for threading ActionPoll
'
Class GetSOSOSData
    Public PC As String

    Public Sub DoIt()
        Try
            ' start RunSOSOS to gather the data (we shell out and wait)
            Shell(My.Settings.RunSOSOS_Path & " " & PC, AppWinStyle.NormalFocus, True)

            ' start ViewSOSOS to look at what we just collected
            Shell(My.Settings.ViewSOSOS_Path & " " & PC, AppWinStyle.NormalFocus, False)
        Catch ex As Exception
            MsgBox("Couldn't successfully run the SOSOS executables" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
End Class

'
' Class for threading ActionExplorer
'
Class LaunchExplorer
    Public PC As String

    Public Sub Doit()
        Try
            Shell(Environment.GetEnvironmentVariable("SystemRoot") & "\explorer.exe /e,\\" & PC & "\C$", AppWinStyle.NormalFocus, False)
        Catch ex As Exception
            MsgBox("Can't start explorer" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

End Class

'
' Class for threading ActionMgmt
'
Class LaunchMMC
    Public PC As String

    Public Sub Doit()
        Try
            Shell(Environment.GetEnvironmentVariable("SystemRoot") & "\System32\mmc.exe " & Environment.GetEnvironmentVariable("SystemRoot") & "\System32\compmgmt.msc", AppWinStyle.NormalFocus, False)
            System.Threading.Thread.Sleep(200)
            System.Windows.Forms.SendKeys.SendWait("%ac" & PC & "~")
        Catch ex As Exception
            MsgBox("Can't start mmc" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
End Class

'
' Class for threading ActionRegistry
'
Class LaunchRegedit
    Public PC As String

    Public Sub Doit()
        Try
            Shell(Environment.GetEnvironmentVariable("SystemRoot") & "\regedit.exe", AppWinStyle.NormalFocus, False)
            System.Threading.Thread.Sleep(200)
            System.Windows.Forms.SendKeys.SendWait("%fc" & PC & "~")
        Catch ex As Exception
            MsgBox("Can't start regedit" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
End Class

'
' Class for threading ActionQuery
'
Class QuerySOSOS
    Public PC As String

    Public Sub Doit()
        Try
            Shell(My.Settings.ViewSOSOS_Path & " " & PC, AppWinStyle.NormalFocus, False)
        Catch ex As Exception
            MsgBox("Couldn't successfully run the ViewSOSOS executable" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
End Class

'
' Class for threading ActionRDP
'
Class LaunchRDP
    Public PC As String

    Public Sub Doit()
        Try
            ' Shell out (and wait for) the Remote Desktop Client.
            Shell(Environment.GetEnvironmentVariable("SystemRoot") & "\System32\mstsc.exe /v:" & PC, AppWinStyle.NormalFocus, True)
        Catch ex As Exception
            MsgBox("Can't start Remote Desktop" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
End Class

'
'
' Class to allow sorting of items by columns
'
Class ListViewItemComparer
    Implements IComparer
    Private col As Integer
    Private order As SortOrder

    Public Sub New()
        col = 0
        order = SortOrder.Ascending
    End Sub

    Public Sub New(ByVal column As Integer, ByVal neworder As SortOrder)
        col = column
        order = neworder
    End Sub

    Public Function Compare(ByVal x As Object, ByVal y As Object) As Integer Implements System.Collections.IComparer.Compare
        Dim returnVal As Integer = -1
        Dim lvi_x, lvi_y As ListViewItem

        lvi_x = CType(x, ListViewItem)
        lvi_y = CType(y, ListViewItem)
        If col >= lvi_x.SubItems.Count Or col >= lvi_y.SubItems.Count Then
            returnVal = [String].Compare(lvi_x.SubItems(0).Text, lvi_y.SubItems(0).Text)
        Else
            returnVal = [String].Compare(lvi_x.SubItems(col).Text, lvi_y.SubItems(col).Text)
        End If

        ' Determine whether the sort order is descending.
        If order = SortOrder.Descending Then
            ' Invert the value returned by String.Compare.
            returnVal *= -1
        End If

        Return returnVal
    End Function

End Class
